@@ -6,6 +6,7 @@ import logging
 import os
 import struct
 import sys
+from datetime import datetime
 from collections import defaultdict
 
 from construct.core import ConstructError
@@ -74,18 +75,17 @@ async def extract_rec(playback, path, select=None):
 def print_info(path):
     """Print basic info."""
     with open(path, 'rb') as handle:
-        summary = Summary(handle, fallback=True)
+        summary = Summary(handle)
         dataset = summary.get_dataset()
-        header = summary.get_header()
         print('-------------')
         print(tabulate([
             ['Path', path],
             ['Duration', mgz.util.convert_to_timestamp(summary.get_duration() / 1000)],
+            ['Played', datetime.utcfromtimestamp(summary.get_played()) if summary.get_played() else None],
             ['Completed', summary.get_completed()],
-            ['Restored', header.initial.restore_time > 0],
+            ['Restored', summary.get_restored()[0]],
             ['Postgame', bool(summary.get_postgame())],
-            ['Objects', bool(summary.get_objects()['objects'])],
-            ['Version', '{} ({}, {}, {})'.format(header.version.name, header.game_version, header.save_version, header.log_version)],
+            ['Version', '{} ({}, {}, {}, {})'.format(*summary.get_version())],
             ['Dataset', '{} {}'.format(dataset['name'], dataset['version'])],
             ['File Hash', summary.get_file_hash()],
             ['Match Hash', summary.get_hash().hexdigest() if summary.get_hash() else None],
