@@ -8,13 +8,12 @@ import zlib
 
 from mgz.util import get_version, Version
 
-
 ZLIB_WBITS = -15
 CLASSES = [b'\x0a', b'\x1e', b'\x46', b'\x50']
 BLOCK_END = b'\x00\x0b'
 REGEXES = {}
 SKIP_OBJECTS = [
-    (b'\x1e\x00\x87\x02', 252) # 647: junk DE object, thousands per file
+    (b'\x1e\x00\x87\x02', 252)  # 647: junk DE object, thousands per file
 ]
 
 
@@ -60,6 +59,8 @@ def hd_string(data):
 def parse_object(data, offset):
     """Parse an object."""
     class_id, object_id, instance_id, pos_x, pos_y = struct.unpack_from('<bxH14xIxff', data, offset)
+    if object_id >= 65535:
+        raise RuntimeError("bad object parse")
     return dict(
         class_id=class_id,
         object_id=object_id,
@@ -116,7 +117,7 @@ def parse_player(header, player_number, num_players):
     type_, *diplomacy, name_length = unpack(f'<bx{num_players}x9i5xh', header)
     name, resources = unpack(f'<{name_length - 1}s2xIx', header)
     header.read(resources * 4)
-    start_x, start_y, civilization_id, color_id = unpack('<xff9xb3xbx',header)
+    start_x, start_y, civilization_id, color_id = unpack('<xff9xb3xbx', header)
     offset = header.tell()
     data = header.read()
     # Skips thousands of bytes that are not easy to parse.
@@ -126,6 +127,8 @@ def parse_player(header, player_number, num_players):
     doppel, end = object_block(data, end, player_number, 2)
     if data[end + 8:end + 10] == BLOCK_END:
         end += 10
+    if data[end:end + 2] == BLOCK_END:
+        end += 2
     header.seek(offset + end)
     return dict(
         number=player_number,
@@ -148,6 +151,8 @@ def parse_lobby(data, version, save):
         data.read(5)
         if save >= 20.06:
             data.read(9)
+        if save >= 26.16:
+            data.read(5)
     data.read(8)
     if version not in (Version.DE, Version.HD):
         data.read(1)
@@ -156,6 +161,8 @@ def parse_lobby(data, version, save):
         data.read(5)
         if save >= 13.13:
             data.read(4)
+        if save >= 25.22:
+            data.read(1)
     chat = []
     for _ in range(0, unpack('<I', data)):
         message = data.read(unpack('<I', data)).strip(b'\x00')
@@ -202,8 +209,8 @@ def parse_map(data, version):
     x2, y2 = unpack('<II', data)
     data.read(x2 * y2 * 4)
     restore_time = unpack('<I', data)
-    if restore_time > 0:
-        raise RuntimeError("restored matches can't be parsed yet")
+    #if restore_time > 0:
+    #    raise RuntimeError("restored matches can't be parsed yet")
     return dict(
         all_visible=all_visible == 1,
         restore_time=restore_time,
@@ -242,10 +249,17 @@ def parse_scenario(data, num_players, version, save):
     map_id, difficulty_id = unpack('<II', data)
     remainder = data.read()
     if version is Version.DE:
-        if save >= 25.06:
-            end = remainder.find(b'\x00\x00\x00\x00\x00\x00\x04\x40')
+        if save >= 26.16:
+            settings_version = 3.0
+        elif save >= 25.22:
+            settings_version = 2.6
+        elif save >= 25.06:
+            settings_version = 2.5
+        elif save >= 13.34:
+            settings_version = 2.4
         else:
-            end = remainder.find(b'\x33\x33\x33\x33\x33\x33\x03\x40')
+            settings_version = 2.2
+        end = remainder.find(struct.pack('<d', settings_version))
         end += 1045
     else:
         end = remainder.find(b'\x9a\x99\x99\x99\x99\x99\xf9\x3f') + 13
@@ -261,6 +275,12 @@ def parse_de(data, version, save, skip=False):
     """Parse DE-specific header."""
     if version is not Version.DE:
         return None
+    build = None
+    if save >= 25.22 and not skip:
+        build = unpack('<I', data)
+    timestamp = None
+    if save >= 26.16 and not skip:
+        timestamp = unpack('<I', data)
     data.read(12)
     dlc_count = unpack('<I', data)
     data.read(dlc_count * 4)
@@ -279,7 +299,9 @@ def parse_de(data, version, save, skip=False):
     for _ in range(8):
         data.read(4)
         color_id = unpack('<i', data)
-        data.read(12)
+        data.read(2)
+        team_id = unpack('<b', data)
+        data.read(9)
         civilization_id = unpack('<I', data)
         de_string(data)
         data.read(1)
@@ -287,24 +309,36 @@ def parse_de(data, version, save, skip=False):
         name = de_string(data)
         data.read(4)
         profile_id, number = unpack('<I4xi', data)
-        data.read(10)
+        if save < 25.22:
+            data.read(8)
+        prefer_random = unpack('b', data)
+        data.read(1)
         if save >= 25.06:
             data.read(8)
         if name:
             players.append(dict(
                 number=number,
                 color_id=color_id,
+                team_id=team_id,
                 name=name,
                 profile_id=profile_id,
-                civilization_id=civilization_id
+                civilization_id=civilization_id,
+                prefer_random=prefer_random == 1
             ))
-    data.read(33)
+    data.read(24)
+    spec_delay = unpack('<I', data)
+    data.read(5)
     for _ in range(23):
         de_string(data)
         c = unpack('<I', data)
         while c in [3, 21, 23, 42, 44, 45, 46, 47]:
             c = unpack('<I', data)
-    data.read(236)
+    if save < 25.22:
+        data.read(236)
+    if save >= 25.22:
+        data.seek(-4, 1)
+        l = unpack('<I', data)
+        data.read(l * 4)
     for _ in range(unpack('<Q', data)):
         data.read(4)
         de_string(data)
@@ -313,6 +347,8 @@ def parse_de(data, version, save, skip=False):
         data.read(8)
     guid = data.read(16)
     lobby = de_string(data)
+    if save >= 25.22:
+        data.read(8)
     mod = de_string(data)
     data.read(33)
     if save >= 20.06:
@@ -321,6 +357,10 @@ def parse_de(data, version, save, skip=False):
         data.read(8)
     if save >= 25.06:
         data.read(21)
+    if save >= 25.22:
+        data.read(4)
+    if save >= 26.16:
+        data.read(8)
     if not skip:
         de_string(data)
         data.read(8)
@@ -334,7 +374,10 @@ def parse_de(data, version, save, skip=False):
         starting_age_id=starting_age_id - 2 if starting_age_id > 0 else 0,
         team_together=not bool(random_positions),
         all_technologies=bool(all_technologies),
-        lock_speed=bool(lock_speed)
+        lock_speed=bool(lock_speed),
+        build=build,
+        timestamp=timestamp,
+        spec_delay=spec_delay
     )
 
 
@@ -417,7 +460,7 @@ def parse_players(header, num_players, version):
     mod = parse_mod(header, num_players, version)
     players = [parse_player(header, number, num_players) for number in range(num_players)]
     for _ in range(num_players):
-        header.read(4)
+        version = unpack('<f', header)
         entries = unpack('<i', header)
         header.read(5 + (entries * 44))
         points = unpack('<i', header)
@@ -451,7 +494,7 @@ def parse(data):
         players, mod = parse_players(header, num_players, version)
         scenario = parse_scenario(header, num_players, version, save)
         lobby = parse_lobby(header, version, save)
-    except (struct.error, zlib.error, AssertionError, MemoryError):
+    except (struct.error, zlib.error, AssertionError, MemoryError, ValueError):
         raise RuntimeError("could not parse")
     return dict(
         version=version,
