@@ -1,7 +1,7 @@
 """Model class definitions."""
 
 from dataclasses import dataclass
-from datetime import timedelta
+from datetime import timedelta, datetime
 from mgz.fast import Action as ActionEnum
 from mgz.util import Version
 
@@ -13,7 +13,7 @@ class Position:
     x: float
     y: float
 
-    def __hash__(self):
+    def hash(self):
         return hash((self.x, self.y))
 
 
@@ -42,6 +42,7 @@ class Player:
     position: Position
     objects: list
     profile_id: int
+    prefer_random: bool = None
     team: list = None
     winner: bool = False
 
@@ -145,6 +146,8 @@ class Match:
     gaia: list
     map: Map
     file: File
+    restored: bool
+    restored_at: timedelta
     speed: str
     speed_id: int
     cheats: bool
@@ -173,6 +176,9 @@ class Match:
     game_version: str
     save_version: float
     log_version: int
+    build_version: int
+    timestamp: datetime
+    spec_delay: timedelta
     hash: str
     actions: list
     inputs: list
